package net.knarcraft.blacksmith.util;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * A test class to test the config helper
 */
public class ConfigHelperTest {

    private static List<String> expectedStringList;

    @BeforeAll
    public static void setUp() {
        expectedStringList = new ArrayList<>();
        for (int i = 1; i < 8; i++) {
            expectedStringList.add(String.valueOf(i));
        }
    }

    @Test
    public void asStringListStringTest() {
        assertEquals(expectedStringList, ConfigHelper.asStringList("1,2,3,4,5,6,7"));
    }

    @Test
    public void asStringListStringListTest() {
        List<String> testValues = new ArrayList<>();
        for (int i = 1; i < 8; i++) {
            testValues.add(String.valueOf(i));
        }
        assertEquals(expectedStringList, ConfigHelper.asStringList(testValues));
    }

    @Test
    public void asStringListIntListTest() {
        List<Integer> testValues = new ArrayList<>();
        for (int i = 1; i < 8; i++) {
            testValues.add(i);
        }
        assertEquals(expectedStringList, ConfigHelper.asStringList(testValues));
    }

    @Test
    public void asDoubleDoubleTest() {
        assertEquals(5.244352345, ConfigHelper.asDouble(5.244352345));
    }

    @Test
    public void asDoubleIntTest() {
        assertEquals(554636d, ConfigHelper.asDouble(554636));
    }

    @Test
    public void asDoubleDoubleStringTest() {
        assertEquals(54.54642388, ConfigHelper.asDouble("54.54642388"));
    }

    @Test
    public void asDoubleIntStringTest() {
        assertEquals(967876554, ConfigHelper.asDouble("967876554"));
    }

    @Test
    public void asDoubleInvalidDoubleStringTest() {
        assertThrows(NumberFormatException.class, () -> ConfigHelper.asDouble("Potato"));
    }

    @Test
    public void asDoubleInvalidObjectTest() {
        assertThrows(ClassCastException.class, () -> ConfigHelper.asDouble(new ArrayList<>()));
    }

    @Test
    public void asBooleanTrueTest() {
        assertTrue(ConfigHelper.asBoolean(true));
    }

    @Test
    public void asBooleanFalseTest() {
        assertFalse(ConfigHelper.asBoolean(false));
    }

    @Test
    public void asBooleanTrueStringTest() {
        assertTrue(ConfigHelper.asBoolean("trUe"));
    }

    @Test
    public void asBooleanFalseStringTest() {
        assertFalse(ConfigHelper.asBoolean("faLse"));
    }

    @Test
    public void asBooleanInvalidStringTest() {
        assertFalse(ConfigHelper.asBoolean("potato"));
    }

    @Test
    public void asBooleanInvalidObjectTest() {
        assertThrows(ClassCastException.class, () -> {
            boolean result = ConfigHelper.asBoolean(new ArrayList<>());
            assertFalse(result);
        });
    }

    @Test
    public void asIntIntTest() {
        assertEquals(5466547, ConfigHelper.asInt(5466547));
    }

    @Test
    public void asIntStringIntTest() {
        assertEquals(976586547, ConfigHelper.asInt("976586547"));
    }

    @Test
    public void asIntInvalidTest() {
        assertThrows(ClassCastException.class, () -> ConfigHelper.asInt(54675467d));
    }

}
